/* constraint.h */

#ifndef INCLUDED_CONSTRAINT_H
#define INCLUDED_CONSTRAINT_H

#include "shared-decls.h"

class Constraint : public RefCounted
{
public:
	typedef std::list< FragmentList > Result_t;

	virtual Result_t
	generateAll () const = 0;

	virtual void
	optimise () const {}
};

class ConstraintMatch : public Constraint
{
public:
	/* FIXME: we don't support masks yet... */
	ConstraintMatch (guint offsetStart,
			 guint offsetEnd) :
		m_offsetStart(offsetStart),
		m_offsetEnd(offsetEnd)
	{
	}

	Result_t
	generateAll () const;


	virtual FragmentFixed*
	generateMatchFrag () const = 0;

private:
	guint m_offsetStart;
	guint m_offsetEnd;
};

class ConstraintMatchString : public ConstraintMatch
{
public:
	ConstraintMatchString (guint offsetStart,
			       guint offsetEnd,
			       std::string matchValue) :
		ConstraintMatch (offsetStart,offsetEnd),
		m_matchValue (matchValue)
	{
	}

	FragmentFixed*
	generateMatchFrag () const;

private:
	std::string m_matchValue;
};

class ConstraintMatchHost16 : public ConstraintMatch
{
public:
	ConstraintMatchHost16 (guint offsetStart,
			       guint offsetEnd,
			       guint16 matchValue) :
		ConstraintMatch (offsetStart,offsetEnd),
		m_matchValue (matchValue)
	{
	}

	FragmentFixed*
	generateMatchFrag () const;

private:
	guint16 m_matchValue;
};

class ConstraintMatchHost32 : public ConstraintMatch
{
public:
	ConstraintMatchHost32 (guint offsetStart,
			       guint offsetEnd,
			       guint32 matchValue) :
		ConstraintMatch (offsetStart,offsetEnd),
		m_matchValue (matchValue)
	{
	}

	FragmentFixed*
	generateMatchFrag () const;

private:
	guint32 m_matchValue;
};

class ConstraintMatchBig16 : public ConstraintMatch
{
public:
	ConstraintMatchBig16 (guint offsetStart,
			       guint offsetEnd,
			       guint16 matchValue) :
		ConstraintMatch (offsetStart,offsetEnd),
		m_matchValue (matchValue)
	{
	}

	FragmentFixed*
	generateMatchFrag () const;

private:
	guint16 m_matchValue;
};

class ConstraintMatchBig32 : public ConstraintMatch
{
public:
	ConstraintMatchBig32 (guint offsetStart,
			       guint offsetEnd,
			       guint32 matchValue) :
		ConstraintMatch (offsetStart,offsetEnd),
		m_matchValue (matchValue)
	{
	}

	FragmentFixed*
	generateMatchFrag () const;

private:
	guint32 m_matchValue;
};

class ConstraintMatchLittle16 : public ConstraintMatch
{
public:
	ConstraintMatchLittle16 (guint offsetStart,
			       guint offsetEnd,
			       guint16 matchValue) :
		ConstraintMatch (offsetStart,offsetEnd),
		m_matchValue (matchValue)
	{
	}

	FragmentFixed*
	generateMatchFrag () const;

private:
	guint16 m_matchValue;
};

class ConstraintMatchLittle32 : public ConstraintMatch
{
public:
	ConstraintMatchLittle32 (guint offsetStart,
			       guint offsetEnd,
			       guint32 matchValue) :
		ConstraintMatch (offsetStart,offsetEnd),
		m_matchValue (matchValue)
	{
	}

	FragmentFixed*
	generateMatchFrag () const;

private:
	guint32 m_matchValue;
};

class ConstraintMatchByte : public ConstraintMatch
{
public:
	ConstraintMatchByte (guint offsetStart,
			       guint offsetEnd,
			       guint16 matchValue) :
		ConstraintMatch (offsetStart,offsetEnd),
		m_matchValue (matchValue)
	{
	}

	FragmentFixed*
	generateMatchFrag () const;

private:
	guint16 m_matchValue;
};

class ConstraintCompound : public Constraint
{
public:
	ConstraintCompound () {}

	ConstraintCompound (Constraint *c0, 
			    Constraint *c1)
	{
		m_children.push_back (c0);
		m_children.push_back (c1);
	}

	void
	addChild (Constraint *child) 
	{
		g_assert (child);
		m_children.push_back (child);
	}

protected:
	typedef std::list< RefCountPtr<Constraint> > ChildContainer_t;
	ChildContainer_t m_children;
};

class ConstraintAnd : public ConstraintCompound
{
public:
	ConstraintAnd () {} 

	ConstraintAnd (Constraint *c0, 
		       Constraint *c1) :
		ConstraintCompound (c0,c1)
	{
	}

	Result_t
	generateAll () const;
};

class ConstraintOr : public ConstraintCompound
{
public:
	Result_t
	generateAll () const;
};

/* Utility function: */
Constraint::Result_t
intersectResults (Constraint::Result_t result0, 
		  Constraint::Result_t result1);


#endif

