/* fragment.h */

#ifndef INCLUDED_FRAGMENT_H
#define INCLUDED_FRAGMENT_H

class FragmentVisitor
{
public:
	void
	visitList (FragmentList& fraglist);

	virtual void
	visitFixed (FragmentFixed& frag) {}

	virtual void
	visitAny (FragmentAny& frag) {}
};

class Fragment : public RefCounted
{
public:
	virtual guint32
	getLength () const = 0;

	virtual void
	acceptVisitor (FragmentVisitor& visitor) = 0;

	virtual Fragment*
	cloneRange (guint32 startByte,
		    guint32 bytesToUse) const = 0;

};

class FragmentFixed : public Fragment
{
public:
	guint32
	getLength () const { return m_bytes.size();}

	void
	acceptVisitor (FragmentVisitor& visitor)
	{
		visitor.visitFixed (*this);
	}

	Fragment*
	cloneRange (guint32 startByte,
		    guint32 bytesToUse) const;

	guint8
	getByte (guint32 index) const
	{
		g_assert (index<m_bytes.size());
		return m_bytes[index];
	}

	void
	addByte (guint8 byte)
	{
		m_bytes.push_back (byte);
	}

private:
	std::vector<guint8> m_bytes;
};

class FragmentAny : public Fragment
{
public:
	FragmentAny (guint32 length) : 
		m_length(length)
	{
	}

	guint32
	getLength () const { return m_length;}

	void
	acceptVisitor (FragmentVisitor& visitor)
	{
		visitor.visitAny (*this);
	}

	Fragment*
	cloneRange (guint32 startByte,
		    guint32 bytesToUse) const;

private:
	guint32 m_length;
};

#endif
