#include "shared-decls.h"
#include "xml-wrapper.h"
#include "xml-wrapper-mime.h"
#include "fragment.h"
#include "constraint.h"
#include "report.h"

/***/

int 
main (int argc, char **argv)
{
	if (argc<2) {
		fprintf (stderr, "Usage:\n");
		fprintf (stderr, "\"mimebreaker /path/tofreedesktop.org.xml\" to search for ambiguities in mime database\n");
		fprintf (stderr, "\"mimebreaker /path/tofreedesktop.org.xml mimetype1 mimetype2 ... mimetypeN\" to generate an example of a file matching all of those mimetypes (if possible) to stdout\n");
		exit (1);
	}

	xmlInitParser ();

	xmlDocPtr xml_doc = xmlParseFile (argv[1]);
	if (NULL==xml_doc) {
		fprintf(stderr, "Couldn't read %s\n", argv[1]);
		exit (1);
	}

	WrapMimeDatabase mimeDatabase(xml_doc);	

	//mimeDatabase.debug();

	if (argc>2) {
		std::vector<WrapMimeType> mimeTypes;
		ReportMakeOneExample report;

		for (int i=2;i<argc;i++) {
			xmlNodePtr node = mimeDatabase.getNodeForMime (argv[i]);
			
			if (NULL==node) {
				fprintf(stderr, "Couldn't find mimetype \"%s\"\n", argv[i]);
				exit(1);
			}
			
			mimeTypes.push_back(WrapMimeType(node));
		}

		Constraint::Result_t result = WrapMimeType::findIntersection (mimeTypes,
									      report);
		if (result.empty()) {
			fprintf(stderr, "No ambiguities found\n");
			exit (1);
		}
	} else 	{
		ReportScreen report;
		//ReportXML report;
		//ReportMakeExamples report ("test-output/output/");
		//ReportFilterTextOnly filtered_report (report);

		try {
			mimeDatabase.findIntersections (report);
			//mimeDatabase.findIntersections (filtered_report);
		} catch (ReportWantsToStop fubar) {}

		//printf("writing report.xml");
		//report.writeToFile ("report.xml");
	}

	xmlCleanupParser ();

	return 0;
}

