/* refcounted.h */
#ifndef INCLUDED_REFCOUNTED_H
#define INCLUDED_REFCOUNTED_H

class RefCounted
{
 public:
	RefCounted() : 
		m_refCount (1)
		{
		}

	void
	incRef () {
		m_refCount++;
	}

	void
	decRef () {
		if (0==--m_refCount) {
			delete this;
		}
	}

 protected:
	RefCounted (const RefCounted &that) :
		m_refCount (1)
		{
		}

	~RefCounted() {}

 private:
	guint32 m_refCount;
};

template <class T> class RefCountPtr
{
public:
	RefCountPtr () : 
		m_rawPtr (NULL) {
	}

	RefCountPtr (T *t) : 
		m_rawPtr (t) {

		if (t) {
			t->incRef ();
		}
	}

	
	RefCountPtr (const RefCountPtr& that) : m_rawPtr (that.m_rawPtr) {
		
		if (m_rawPtr) {
			m_rawPtr->incRef ();
		}
	}

	RefCountPtr& operator= (const RefCountPtr& that) {
		if (that.m_rawPtr) {
			that.m_rawPtr->incRef ();
		}
		if (m_rawPtr) {
			m_rawPtr->decRef ();
		}

		m_rawPtr = that.m_rawPtr;
	}

	~RefCountPtr () {
		if (m_rawPtr) {
			m_rawPtr->decRef ();
		}
	}

	T* operator-> () {
		g_assert (m_rawPtr);
		return m_rawPtr;
	}

	const T* operator-> () const {
		g_assert (m_rawPtr);
		return m_rawPtr;
	}

	T& operator* () {
		g_assert (m_rawPtr);
		return *m_rawPtr;
	}

	const T& operator* () const {
		g_assert (m_rawPtr);
		return *m_rawPtr;
	}

	bool isNonNull () const {return NULL!=m_rawPtr;}

 private:

	T *m_rawPtr; 
};

#endif 
