#include "report.h"

class FragmentVisitorDump : public FragmentVisitor
{
public:
	void
	visitFixed (FragmentFixed& frag)
	{
		for (int i=0;i<frag.getLength ();i++) {
			printf("0x%2X ", frag.getByte (i));
		}
	}

	void
	visitAny (FragmentAny& frag)
	{
		for (int i=0;i<frag.getLength ();i++) {
			printf("0x?? ");
		}
	}

private:

};

void
dumpResults (Constraint::Result_t results)
{
	for (Constraint::Result_t::iterator iter = results.begin(); iter!=results.end(); iter++) {

		FragmentVisitorDump visitor;

		visitor.visitList (*iter);		
	}
	printf("\n");
}

// class Report
bool 
Report::
considerMimeType(WrapMimeType mimeType) const
{
	// Various policies are possible

#if 0
	if (matchNames() && !mimeType.hasGlob()) {
		return false;
	}
#endif

	if (requireMagic() && !mimeType.hasMagic()) {
		return false;
	}

	// If a mimetype has neither a <match> nor a <glob> it probably shouldn't appear:
	if (mimeType.hasConstraints()) {
		return true;
	}

	return false;
}


// class ReportScreen : public Report
void
ReportScreen::
onIntersections (std::vector<WrapMimeType> mimeTypes,  
		 Constraint::Result_t result)
{
	{
		printf ("Got %i-way intersections of (", mimeTypes.size());

		for (std::vector<WrapMimeType>::iterator iter = mimeTypes.begin(); iter!=mimeTypes.end(); iter++) {
			printf("\"%s\" ", iter->asString().c_str());
		}
		printf (")");
	}

	if (mimeTypes.size()>=8) {
		for (Constraint::Result_t::iterator iter = result.begin(); iter!=result.end(); iter++) {
			dumpResults (result);
		}		
		
		throw ReportWantsToStop();
	}
}

// class ReportXML : public Report
ReportXML::
ReportXML ()
{
	m_xmlDoc = xmlNewDoc ((xmlChar*)"1.0");

	m_nodeRoot = NodeWrapper((xmlNodePtr)m_xmlDoc).addChild ("sniff-report");
}

ReportXML::
~ReportXML ()
{
}

void
ReportXML::
onIntersections (std::vector<WrapMimeType> mimeTypes,  
		 Constraint::Result_t result)
{
	/* 10 matches */
	if (mimeTypes.size()>=10) {

	xmlNodePtr t_nodeSniffIntersection = NodeWrapper(m_nodeRoot).addChild ("sniff-intersection");
	xmlNodePtr t_nodeMimeList = NodeWrapper(t_nodeSniffIntersection).addChild ("mime-list");
	xmlNodePtr t_nodeIntersectionList = NodeWrapper(t_nodeSniffIntersection).addChild ("intersection-list");

	{
		for (std::vector<WrapMimeType>::iterator iter = mimeTypes.begin(); iter!=mimeTypes.end(); iter++) {
			xmlNodePtr t_nodeMimeType = NodeWrapper(t_nodeMimeList).addChild ("mime-type");
			NodeWrapper(t_nodeMimeType).addAttr ("name", iter->asString());
		}
	}

	{
		for (Constraint::Result_t::iterator iter = result.begin(); iter!=result.end(); iter++) {
			xmlNodePtr t_nodeIntersection = NodeWrapper(t_nodeIntersectionList).addChild ("intersection");

			guint32 t_byteOffset = 0;

			for (FragmentList::iterator frag_iter = iter->begin(); frag_iter!=iter->end(); frag_iter++) {
				RefCountPtr<Fragment> frag = *frag_iter;
				RefCountPtr<FragmentFixed> fixed = dynamic_cast<FragmentFixed*>(&*frag);

				if (fixed.isNonNull ()) {
					xmlNodePtr t_nodeFrag = NodeWrapper(t_nodeIntersection).addChild ("fragment-fixed");

					for (int i=0;i<frag->getLength();i++) {
						xmlNodePtr t_nodeByte = NodeWrapper(t_nodeFrag).addChild ("byte");
						guint8 t_byte = fixed->getByte (i);
						NodeWrapper(t_nodeByte).addAttr ("offset-decimal", t_byteOffset);
						NodeWrapper(t_nodeByte).addAttrAsHex ("offset-hex", 8, t_byteOffset);
						t_byteOffset++;

						NodeWrapper(t_nodeByte).addAttr ("decimal", t_byte);
						NodeWrapper(t_nodeByte).addAttrAsHex ("hex", 2, t_byte);

						if (g_ascii_isprint (t_byte)) {
							gchar *str = g_strdup_printf ("%c", t_byte);
							NodeWrapper(t_nodeByte).addAttr ("ascii", str);
							g_free (str);
						}
					}
				} else {
					xmlNodePtr t_nodeFrag = NodeWrapper(t_nodeIntersection).addChild ("fragment-any");
					NodeWrapper(t_nodeFrag).addAttr ("offset-decimal", t_byteOffset);
					NodeWrapper(t_nodeFrag).addAttrAsHex ("offset-hex", 8, t_byteOffset);
					NodeWrapper(t_nodeFrag).addAttr ("length", frag->getLength ());

					t_byteOffset += frag->getLength ();
				}
			}
		}
	}

	throw ReportWantsToStop();
	}
}

class FragmentVisitorWriteExample : public FragmentVisitor
{
public:
	FragmentVisitorWriteExample (FILE *file)
	{
		m_pFile = file;
	}

	~FragmentVisitorWriteExample() {
	}

	void
	visitFixed (FragmentFixed& frag)
	{
		for (int i=0;i<frag.getLength ();i++) {
			guint8 value = frag.getByte (i);
			fwrite (&value, 1, 1, m_pFile);
		}
	}

	void
	visitAny (FragmentAny& frag)
	{
		for (int i=0;i<frag.getLength ();i++) {
			guint8 value = '*';
			fwrite (&value, 1, 1, m_pFile);
		}
	}

private:
	FILE *m_pFile;

};

void
writeExampleFile (std::string filename,
		  FragmentList& list)
{
	FILE * pFile = fopen (filename.c_str(), "wb");

	FragmentVisitorWriteExample visitor(pFile);

	visitor.visitList (list);

	fclose (pFile);
}

// class ReportMakeExamples : public Report
void
ReportMakeExamples::
onIntersections (std::vector<WrapMimeType> mimeTypes,  
		 Constraint::Result_t result)
{
	const bool only_one_of_each = true;

	std::string dirname;

	{
		for (std::vector<WrapMimeType>::iterator iter = mimeTypes.begin(); iter!=mimeTypes.end(); iter++) {
			if (iter!=mimeTypes.begin()) {
				dirname += "_and_";
			}

			dirname += iter->asString();
		}
	}

	gchar *input = g_strdup (dirname.c_str());
	g_strdelimit (input,
		      "/",
		      '-');
	dirname = input;
	g_free (input);

	dirname = m_path + dirname;

	g_message ("dirname = %s", dirname.c_str());

	/* For now, these aren't directory names, but simple prefixes to use: */

	{
		int i = 1;
		for (Constraint::Result_t::iterator iter = result.begin(); iter!=result.end(); iter++) {
			gchar *filename;

			if (only_one_of_each) {
				writeExampleFile (dirname,
						  *iter);
			} else {
				filename = g_strdup_printf ("%s%03i", dirname.c_str(), i++);

				writeExampleFile (filename,
						  *iter);
				
				g_free (filename);
			} 

			// For the sake of my hard drive, only output the first for each mime-type combo:
			if (only_one_of_each) {
				break;
			}
		}
	}
}

// class ReportMakeOneExample : public Report
void
ReportMakeOneExample::
onIntersections (std::vector<WrapMimeType> mimeTypes,  
		 Constraint::Result_t result)
{
	if (result.empty()) {
		fprintf(stderr, "No ambiguities found");
		exit (1);
	}

	FragmentVisitorWriteExample visitor(stdout);
	visitor.visitList (*result.begin());
}


// class ReportFilter : public Report
void
ReportFilter::
onIntersections (std::vector<WrapMimeType> mimeTypes,  
		 Constraint::Result_t result)
{
	Constraint::Result_t filtered;
	for (Constraint::Result_t::iterator iter = result.begin(); iter!=result.end(); iter++) {
		if (includeThisResult (*iter)) {
			filtered.push_back (*iter);
		}
	}

	if (!filtered.empty ()) {
		m_innerReport.onIntersections (mimeTypes,  
					       filtered);
	}
}




