/* report.h */

#ifndef INCLUDED_REPORT_H
#define INCLUDED_REPORT_H

#include "shared-decls.h"
#include "constraint.h"
#include "fragment.h"
#include "xml-wrapper-mime.h"

class Report
{
public:
	virtual ~Report() {}

	int arity() const {return 2;}
	
	bool recurse() const { return false; }
	bool matchNames() const { return true; }
	bool requireMagic() const {return true; }

	bool considerMimeType(WrapMimeType mimeType) const;

	virtual void
	onIntersections (std::vector<WrapMimeType> mimeTypes,  
			 Constraint::Result_t result) = 0;
private:
};

class ReportScreen : public Report
{
public:
	void
	onIntersections (std::vector<WrapMimeType> mimeTypes,  
			 Constraint::Result_t result);
};

class ReportXML : public Report
{
public:
	ReportXML ();
	~ReportXML ();

	void
	onIntersections (std::vector<WrapMimeType> mimeTypes,  
			 Constraint::Result_t result);

	void
	writeToFile (std::string filename) const
	{
		DocWrapper (m_xmlDoc).writeToFile (filename);
	}

private:
	xmlDocPtr m_xmlDoc;
	xmlNodePtr m_nodeRoot;
};

class ReportMakeExamples : public Report
{
public:
	ReportMakeExamples (std::string path) : 
		m_path (path)
	{
	}

	void
	onIntersections (std::vector<WrapMimeType> mimeTypes,  
			 Constraint::Result_t result);

private:
	std::string m_path;
};

class ReportMakeOneExample : public Report
{
public:
	void
	onIntersections (std::vector<WrapMimeType> mimeTypes,  
			 Constraint::Result_t result);

private:
};

class ReportFilter : public Report
{
public:
	ReportFilter (Report& innerReport) :
		m_innerReport (innerReport)
	{
	}

	void
	onIntersections (std::vector<WrapMimeType> mimeTypes,  
			 Constraint::Result_t result);
	
	virtual bool
	includeThisResult (const FragmentList& fraglist) = 0;

private:
	Report& m_innerReport;
};

#if 0
class ReportFilterMostIntersections : public ReportFilter
{
public:
private:
};
#endif

class ReportFilterTextOnly : public ReportFilter
{
public:
	ReportFilterTextOnly (Report& innerReport) :
		ReportFilter (innerReport)
	{
	}

	bool
	includeThisResult (const FragmentList& fraglist)
	{
		for (FragmentList::const_iterator frag_iter = fraglist.begin(); frag_iter!=fraglist.end(); frag_iter++) {
			RefCountPtr<Fragment> frag = *frag_iter;
			FragmentFixed *fixed = dynamic_cast<FragmentFixed*>(&*frag);
			
			if (fixed) {
				for (int i=0;i<frag->getLength();i++) {
					if (!g_ascii_isprint (fixed->getByte (i))) {
						return false;
					}
				}
			}
		}
		
		return true;
	}
};

// Exception:
class ReportWantsToStop {};

#endif
