/* xml-wrapper-mime.h */

#ifndef INCLUDED_XML_WRAPPER_MIME_H
#define INCLUDED_XML_WRAPPER_MIME_H

#include "xml-wrapper.h"
#include "constraint.h"

class WrapMimeDatabase : public DocWrapper
{
public:
	WrapMimeDatabase (xmlDocPtr xml_doc) :
		DocWrapper (xml_doc)
	{
		m_nodeMimeInfo = locateChildElement ("mime-info");
	}

	xmlNodePtr
	getNodeForMime (std::string mimeType) const;

	void
	forEachMimeType ( void (*callback) (const WrapMimeType& mimeType));

	void
	debug () const;
	
	void
	findIntersections (Report& report) const;

	void
	findIntersectionsRecursive (std::vector<WrapMimeType> intersectingMimeTypes,
				    Constraint::Result_t inputResults,
				    Report &report) const;

private:
	xmlNodePtr m_nodeMimeInfo;
};

class WrapMimeType : public NodeWrapper
{
public:
	WrapMimeType (xmlNodePtr xml_node) : 
		NodeWrapper (xml_node)
	{
	}

	std::string
	asString () const;

	bool
	hasMagic () const;

	// Is this mimetype constrained in any way?  Does it either specify a <glob> or <match> pattern?  Otherwise anything could satify it...
	bool
	hasConstraints () const;

	Constraint*
	makeConstraint () const;

	static Constraint::Result_t
	findIntersection (std::vector<WrapMimeType> mimeTypes,
			  Report& report);

	Constraint::Result_t
	generateAll () const;

	Constraint::Result_t
	findIntersectionOptimised (std::vector<WrapMimeType> mimeTypes,
				   Constraint::Result_t inputResults) const;
};

class WrapMimeMagic : public NodeWrapper

{
public:
	WrapMimeMagic (xmlNodePtr xml_node) : 
		NodeWrapper (xml_node)
	{
	}

	Constraint*
	makeConstraint () const;

	int
	getPriority () const;

};

class WrapMimeMatch : public NodeWrapper
{
public:
	WrapMimeMatch (xmlNodePtr xml_node) :
		NodeWrapper (xml_node)
	{
	}

	Constraint*
	makeConstraint () const;


private:
	static void
	calcOffsets (std::string attrOffset,
		     guint &offsetStart,
		     guint &offsetEnd);

};

#endif
