#include "xml-wrapper.h"

// class NodeWrapper
bool
NodeWrapper::
isElement (std::string localName) const
{
	if (m_xmlNode->type==XML_ELEMENT_NODE) {
		if (localName == (const char*)m_xmlNode->name) {
			return true;
		}
	}
		
	return false;
}

bool
NodeWrapper::
hasChild (std::string localName) const
{
	return locateChildElement (localName);
}


xmlNodePtr
NodeWrapper::
locateChildElement (std::string localName) const
{
	xmlNodePtr iter;

	for (iter=m_xmlNode->children;iter;iter=iter->next) {
		if (NodeWrapper(iter).isElement (localName)) {
			return iter;
		}
	}

	return NULL;
}

std::string
NodeWrapper::
getAttrValue (std::string attrName) const
{
	xmlChar *value = xmlGetProp (m_xmlNode,
				     BAD_CAST attrName.c_str());

	if (value) {
		return std::string ((const char*)value);
	} else {
		return "";
	}
}

int
NodeWrapper::
getAttrAsInt (std::string attrName,
	      int defaultValue) const
{
	int value = defaultValue;
	std::string attrString = getAttrValue (attrName);

	sscanf (attrString.c_str(), "%i", &value);

	return value;
}


xmlNodePtr
NodeWrapper::
addChild (std::string localName) const
{
	return xmlNewChild (m_xmlNode,
			    NULL,
			    (xmlChar*)localName.c_str(),
			    NULL);
}

void
NodeWrapper::
addAttr (std::string attrName,
	 std::string attrValue) const
{
	xmlNewProp (m_xmlNode,
		    (xmlChar*)attrName.c_str(),
		    (xmlChar*)attrValue.c_str());
}

void
NodeWrapper::
addAttr (std::string attrName,
	 int attrValue) const
{
	gchar *tmp = g_strdup_printf ("%i", attrValue);

	addAttr (attrName,
		 tmp);

	g_free (tmp);
}

void
NodeWrapper::
addAttrAsHex (std::string attrName,
	      guint8 numDigits,
	      guint32 attrValue) const
{
	gchar *str;

	switch (numDigits) {
	default: g_assert_not_reached ();
	case 2: 
		g_assert (attrValue<256);
		str = g_strdup_printf ("%02X", attrValue);
		break;
	case 8:
		str = g_strdup_printf ("%08X", attrValue);
		break;
	}

	g_assert (str);
	addAttr (attrName,
		 str);

	g_free (str);
}

// class DocWrapper : public NodeWrapper
void
DocWrapper::
writeToFile (std::string filename) const
{
	xmlSaveFile (filename.c_str (),
		     getDocPtr ());
}
