/* xml-wrapper.h */

#ifndef INCLUDED_XML_WRAPPER_H
#define INCLUDED_XML_WRAPPER_H

#include "shared-decls.h"

class NodeWrapper
{
public:
	NodeWrapper(xmlNodePtr xml_node) : 
		m_xmlNode (xml_node)
	{
	}

	bool
	isElement (std::string localName) const;

	xmlNodePtr
	getNode () const {return m_xmlNode;}
	
	bool
	hasChild (std::string localName) const;

	xmlNodePtr
	locateChildElement (std::string localName) const;

	std::string
	getAttrValue (std::string attrName) const;

	int
	getAttrAsInt (std::string attrName,
		      int defaultValue) const;
	
	/* Modifying document: */
	xmlNodePtr
	addChild (std::string localName) const;

	void
	addAttr (std::string attrName,
		 std::string attrValue) const;

	void
	addAttr (std::string attrName,
		 int attrValue) const;

	void
	addAttrAsHex (std::string attrName,
		      guint8 numDigits,
		      guint32 attrValue) const;

protected:
	xmlNodePtr m_xmlNode;
};

class DocWrapper : public NodeWrapper
{
public:
	DocWrapper (xmlDocPtr xml_doc) :
		NodeWrapper ((xmlNodePtr)xml_doc)
	{
	}

	xmlDocPtr
	getDocPtr () const 
	{
		return (xmlDocPtr)m_xmlNode;
	}

	void
	writeToFile (std::string filename) const;
};


#endif
